#!/bin/bash

######
# 
# Script for building GT4 Web services
# developed under the Eclipse IDE
#
# by Marco Lackovic 
# Dept. of Electronics, Computer Science and Systems
# University of Calabria, Italy
# http://grid.deis.unical.it/lackovic/
#
# Version 1.3, May 2010
#
######

# Note: run this file with root privileges

if [ -z "${GLOBUS_LOCATION}" ]; then
    echo "ERROR: environment variable GLOBUS_LOCATION not defined" 1>&2
    exit
fi

if [ ! -d "${GLOBUS_LOCATION}" ]; then
    echo "ERROR: invalid GLOBUS_LOCATION set: $GLOBUS_LOCATION" 1>&2
    exit
fi

# ----------------------------------------------------

# SERVICE: Service id (the first parameter in the build.mappings file)
# USERNAME: Username
# PROJECT: Eclipse project name 
# PACKAGE_ROOT: Package root

SERVICE=epms
USERNAME=lackovic
PROJECT=EPMS
PROJECT2=DIS3GNO
PACKAGE_ROOT=kgrid

# ----------------------------------------------------

WORKSPACE=/home/${USERNAME}/workspace/$PROJECT
WORKSPACE2=/home/${USERNAME}/workspace/$PROJECT2
DEST=/home/globus/build/$SERVICE
PACKAGE_PATH=$PACKAGE_ROOT/$SERVICE
GAR=${PACKAGE_ROOT}_${SERVICE}_service

# ----------------------------------------------------

if [ ! -d "$WORKSPACE" ]; then
    echo "ERROR: directory $WORKSPACE does not exist" 1>&2
    exit
fi
if [ -d "$DEST" ]; then
	echo "Deleting old build files on $DEST..." 1>&2
    rm -rf $DEST
fi

echo
echo ------------------------------------------------------------
echo
echo Building $SERVICE service
echo

# ----------------------------------------------------

echo Directories creation...
echo

mkdir -p $DEST/schema/$PACKAGE_PATH/
mkdir -p $DEST/$PACKAGE_PATH/service/etc/
mkdir -p $DEST/$PACKAGE_PATH/service/impl/
mkdir -p $DEST/lib/

# ----------------------------------------------------

echo Copying from user workspace to globus space...
echo 

cd $DEST/
cp $WORKSPACE/src/$PACKAGE_PATH/service/schema/* schema/$PACKAGE_PATH/
cp -r $WORKSPACE/src/$PACKAGE_PATH/* $PACKAGE_PATH/
cp $WORKSPACE/lib/*.jar lib/
cp $WORKSPACE/build/globus-build-service.sh .
cp $WORKSPACE/build/build.xml .
tr -d '\r$' < $WORKSPACE/build/build.mappings > build.mappings
tr -d '\r$' < $WORKSPACE/build/namespace2package.mappings > namespace2package.mappings
chown -R globus. $DEST/

# ----------------------------------------------------

echo Changing scripts permissions...
echo

chmod +x *.sh

# ----------------------------------------------------

$GLOBUS_LOCATION/bin/globus-undeploy-gar $GAR

# ----------------------------------------------------

for i in $DEST/lib/*.jar
do
  CLASSPATH=$CLASSPATH:"$i"
done

export CLASSPATH

./globus-build-service.sh $SERVICE

echo 

# ----------------------------------------------------

if [ ! -s "${GAR}.gar" ]; then
    echo "ERROR: file ${GAR}.gar does not exist" 1>&2
    exit
fi

$GLOBUS_LOCATION/bin/globus-deploy-gar ./${GAR}.gar
echo

# ----------------------------------------------------

echo Changing owner of globus files...
echo

chown -R globus. $DEST/
chown -R globus. $GLOBUS_LOCATION/lib/${GAR}*
chown -R globus. $GLOBUS_LOCATION/etc/${GAR}*
chown -R globus. $GLOBUS_LOCATION/share/schema/${PACKAGE_ROOT}*
chown -R globus. $GLOBUS_LOCATION/share/globus_wsrf_common/tmp
chown -R globus. $GLOBUS_LOCATION/etc/globus_packages/${GAR}*

# ----------------------------------------------------

echo Copying stubs to workspace...
echo

cp $GLOBUS_LOCATION/lib/${GAR}_stubs.jar $WORKSPACE/lib/
cp $GLOBUS_LOCATION/lib/${GAR}_stubs.jar $WORKSPACE2/lib/
cp $DEST/*.gar $WORKSPACE/build/

# ----------------------------------------------------

echo Changing owner of user files...
echo

chown -R $USERNAME. $WORKSPACE/lib/*
chown -R $USERNAME. $WORKSPACE2/lib/*
chown -R $USERNAME. $WORKSPACE/build/*
cd

# ----------------------------------------------------

echo "Please REFRESH your Eclipse project lib directory."
echo
date
