/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.epms.client;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.security.cert.X509Certificate;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import kgrid.epms.service.impl.Consts;
import kgrid.epms.service.impl.Utils;
import kgrid.epms.stubs.EPMSFactory.CreateResource;
import kgrid.epms.stubs.EPMSFactory.CreateResourceResponse;
import kgrid.epms.stubs.EPMSFactory.EPMSFactoryPortType;
import kgrid.epms.stubs.EPMSFactory.service.EPMSFactoryServiceAddressingLocator;
import kgrid.epms.stubs.EPMSService.EPMSPortType;
import kgrid.epms.stubs.EPMSService.ExecutionParameters;
import kgrid.epms.stubs.EPMSService.Parameters;
import kgrid.epms.stubs.EPMSService.service.EPMSServiceAddressingLocator;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.xml.serialize.DOMSerializer;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.globus.axis.util.Util;
import org.globus.delegation.DelegationUtil;
import org.globus.gsi.GSIConstants;
import org.globus.gsi.GlobusCredential;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;
import org.globus.wsrf.utils.AddressingUtils;
import org.w3c.dom.Document;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 */
public class EPMSClient implements Consts {

	static {
		Util.registerTransport();
	}

	public static void printDocument(Document doc) {
		try {
			PrintWriter pw = new PrintWriter(System.out);
			DOMSerializer ser = new XMLSerializer(pw, new OutputFormat(doc,
					"UTF-8", true));
			ser.serialize(doc);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}// printDocument

	public static void main(String[] args) {

		EPMSFactoryServiceAddressingLocator factoryLocator = new EPMSFactoryServiceAddressingLocator();
		EPMSServiceAddressingLocator instanceLocator = new EPMSServiceAddressingLocator();

		try {

			String factoryURI = args[0] + "/" + EPMS_FACTORY;
			EndpointReferenceType factoryEPR, instanceEPR;
			EPMSFactoryPortType epmsFactory;
			EPMSPortType epms;
			System.out.println("Get Factory Port Type");
			// 1: get Factory Port Type
			factoryEPR = new EndpointReferenceType();
			factoryEPR.setAddress(new Address(factoryURI));
			epmsFactory = factoryLocator.getEPMSFactoryPortTypePort(factoryEPR);
			System.out
					.println("Create resource and get endpoint reference of WS-Resource");

			// 2: Create resource and get endpoint reference of WS-Resource.
			// This resource is our "instance".

			CreateResourceResponse createResponse = epmsFactory
					.createResource(new CreateResource());
			instanceEPR = createResponse.getEndpointReference();

			// 3
			System.out.println("Invocation method");
			epms = instanceLocator.getEPMSPortTypePort(instanceEPR);

			EndpointReferenceType epr = delegateCredential(Utils.HOST_IP,
					CONTAINER_PORT);

			System.out.println("epr=" + epr.toString());

			// System.out.println(d.getChildNodes().item(0).getNodeName());
			// ep.setExecutionPlanDoc(new DataHandler(new
			// FileDataSource("/home/globus/TEST/abstract_model0.xml")));
			// ep.setEprCredential(epr);

			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = dbf.newDocumentBuilder();
			Document document = builder.parse(new File(args[1]));
			// ("/home/cesario/TEST/abstract_model.xml");
			// "/home/marino/Desktop/TEST/abstract_model0.xml");
			// d.normalize();

			// printDocument(d);

			Parameters ep = new Parameters(document, epr);
			// ExecutionParameters ep =
			// new
			// ExecutionParameters(delegateCredential(delegation_host,"8443"),d);

			// ExecutionResults ris=epms.submitKApplication(ep);
			System.out.println("Sto per invocare il servizio...");
			Object o = epms.submitKApplication(ep);
			if (o instanceof ExecutionParameters) {
				ExecutionParameters ris = (ExecutionParameters) o;
				System.out
						.println("Il risultato è un oggetto ExecutionResults");
				Object o2 = ris.getExecutionPlanDoc();
				if (o2 instanceof Document) {
					Document executionPlanDoc = (Document) o2;
					System.out
							.println("Dall'ExecutionResults ottengo un oggetto Document");
					// Document executionPlanDoc=ris.getExecutionPlanDoc();
					// executionPlanDoc.normalize();
					// System.out.println(executionPlanDoc);
					printDocument(executionPlanDoc);
					System.out.println("ok");
				} else
					System.out
							.println("Dall'ExecutionResults ottengo un oggetto "
									+ o2.getClass().getName());

			} else
				System.out.println("Il risultato è un oggetto "
						+ o.getClass().getName());

		} catch (Exception e) {
			e.printStackTrace();
		}

	}// main

	// Contact the Delegation Factory Service and get an EPR
	// for the Delegation Resource that contains your delegated credential.
	private static EndpointReferenceType delegateCredential(String host,
			int port) throws Exception {
		ClientSecurityDescriptor desc = getClientSecDesc();
		GlobusCredential credential = GlobusCredential.getDefaultCredential();
		String factoryUrl = ("https://" + host + ":" + port + "/" + DELEGATION_FACTORY);
		int terminationTime = 60;
		int lifetime = terminationTime * 60; // lifetime in seconds
		EndpointReferenceType delegEpr = AddressingUtils
				.createEndpointReference(factoryUrl, null);
		X509Certificate[] certsToDelegateOn = DelegationUtil
				.getCertificateChainRP(delegEpr, desc);
		X509Certificate certToSign = certsToDelegateOn[0];
		return DelegationUtil.delegate(factoryUrl, credential, certToSign,
				lifetime, false, desc);
	}

	private static ClientSecurityDescriptor getClientSecDesc() {
		ClientSecurityDescriptor desc = new ClientSecurityDescriptor();
		Authorization authzVal = HostAuthorization.getInstance();
		Object authVal = GSIConstants.SIGNATURE;
		desc.setGSITransport((Integer) authVal);
		Util.registerTransport();
		desc.setAuthz(authzVal);
		return desc;
	}

}
