/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.epms.service.impl;

import java.net.URL;
import java.rmi.RemoteException;

import kgrid.epms.stubs.EPMSFactory.CreateResource;
import kgrid.epms.stubs.EPMSFactory.CreateResourceResponse;

import org.apache.axis.MessageContext;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.wsrf.ResourceContext;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.container.ServiceHost;
import org.globus.wsrf.utils.AddressingUtils;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class EPMSFactoryService {

    /**
     * Implementation of createResource operation
     */
    public CreateResourceResponse createResource(CreateResource request)
            throws RemoteException {
        ResourceContext resContext = null;
        EPMSResourceHome resHome = null;
        ResourceKey resKey = null;

        try {
            resContext = ResourceContext.getResourceContext();
            resHome = (EPMSResourceHome) resContext.getResourceHome();
            resKey = resHome.create();
        } catch (Exception e) {
            throw new RemoteException(e.getMessage(), e);
        }
        EndpointReferenceType epr = null;

        // We construct the instance's endpoint reference. The instance's
        // service path can be found in the WSDD file as a parameter.
        try {
            URL baseURL = ServiceHost.getBaseURL();
            String instanceService = (String) MessageContext
                    .getCurrentContext().getService().getOption("instance");
            String instanceURI = baseURL.toString() + instanceService;
            // The endpoint reference includes the instance's URI and the
            // resource key
            epr = AddressingUtils.createEndpointReference(instanceURI, resKey);
        } catch (Exception e) {
            throw new RemoteException(e.getMessage(), e);
        }

        // Finally, return the endpoint reference in a CreateResourceResponse
        CreateResourceResponse response = new CreateResourceResponse();
        response.setEndpointReference(epr);
        return response;
    }

}
