/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.epms.service.impl;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.xml.namespace.QName;

import kgrid.epms.service.impl.tag.DataMiningSoftware;
import kgrid.epms.service.impl.tag.Dataset;
import kgrid.epms.service.impl.tag.Edge;
import kgrid.raems.stubs.RAEMSService.RAEMSPortType;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.axis.util.Util;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.WSNConstants;
import org.globus.wsrf.core.notification.ResourcePropertyValueChangeNotificationElementType;
import org.oasis.wsn.Subscribe;
import org.oasis.wsn.TopicExpressionType;
import org.oasis.wsrf.properties.ResourcePropertyValueChangeNotificationType;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class EPMSListener implements NotifyCallback, Consts {
	static {
		Util.registerTransport();
	}
	private RAEMSPortType portType;
	private EPMSResource resource;
	private HashMap<String, Vector> taskNodeMap = null;
	private HashMap<String, Edge> infoEdgeMap = null;

	/*
	 * Contains the abstract execution plan xml elements mapped with their own
	 * ids (e.g. "e2" --> "<Task name="sub_job1">...</Task>", etc)
	 */
	private HashMap<String, Object> elementMap = null;

	// Contains all the workflow elements ids (e.g. "n5", "e20", etc)
	private Set<String> idSet = null;

	private LinkedList<String> downloadStarted = new LinkedList<String>();
	private LinkedList<String> downloadStopped = new LinkedList<String>();

	public EPMSListener(RAEMSPortType rm, EPMSResource resource,
			HashMap<String, Vector> mapTaskNode,
			HashMap<String, Object> mapIdNode, HashMap<String, Edge> infoEdge) {
		try {
			portType = rm;
			this.resource = resource;
			this.taskNodeMap = mapTaskNode;
			this.elementMap = mapIdNode;
			this.infoEdgeMap = infoEdge;
			idSet = mapIdNode.keySet();
			subscribeToRAEMSModelTopic(this, portType);
		} catch (Throwable e1) {
			e1.printStackTrace();
		}
	}

	public synchronized void deliver(List topicPath,
			EndpointReferenceType producer, Object message) {
		synchronized (this) {
			ResourcePropertyValueChangeNotificationElementType notif_elem;
			ResourcePropertyValueChangeNotificationType notif;

			notif_elem = (ResourcePropertyValueChangeNotificationElementType) message;
			notif = notif_elem.getResourcePropertyValueChangeNotification();

			if (notif != null) {
				for (MessageElement me : notif.getNewValue().get_any()) {
					Object temp = elaborate(me.getValue());
					// if(temp==null)
					// resource.setLastOp(me.getValue());
					// else
					if (temp != null) {
						if (temp instanceof String) {
							String notify = (String) temp;
							resource.setLastOp(notify);
						} else if (temp instanceof Vector) {
							Vector<String> notifys = (Vector<String>) temp;
							for (int i = 0; i < notifys.size(); i++) {
								String notify = notifys.get(i);
								resource.setLastOp(notify);
							}
						}
					}
				}
			}
			this.notify();
		}
	}

	private Object elaborate(String source) {

		if (source.contains("#PREPROCESSING")) {
			int end = source.indexOf("$", 7);
			if (source.contains(":Download started$")) {
				String subject = source.substring(end + 2);
				String id = resolveDownload(subject, downloadStarted);
				String ris = "#STATE:" + id + "$" + subject
						+ " download started";
				return ris;
			}
			if (source.contains(":Download completed$")) {
				String subject = source.substring(end + 2);
				String id = resolveDownload(subject, downloadStopped);
				String ris = "#STATE:" + id + "$" + subject
						+ " download completed";
				return ris;
			}
			String id = resolveList(source);
			int start = source.indexOf(":", end);
			if (id == null) {
				id = resolveList("");
			}
			String ris = "#STATE:" + id + "$available resources="
					+ source.substring(start + 1);
			return ris;
		} else if (source.contains("#STATE")) {
			if (source.contains("Source") && source.contains("Destination")
					|| source.contains("ACTIVE EDGE"))
				return resolveEdge(source);
			int start = source.indexOf(":");
			int stop = source.indexOf("$", 7);
			String name = source.substring(start + 1, stop);
			Vector<String> nodes = taskNodeMap.get(name);
			Vector<String> notifys = new Vector<String>();
			if (nodes != null) {
				for (int i = 0; i < nodes.size(); i++) {
					String id = nodes.get(i);
					String notify = "#STATE:" + id + source.substring(stop);
					// TODO (-4-) EPMS: add notification of workflow termination
					notifys.add(notify);
				}
				return notifys;
			}
		} else if (source.contains("#ERROR")) {
			int end = source.indexOf("$", 7);
			String cause = source.substring(end + 1);
			int start = source.indexOf(":") + 1;
			int stop = source.indexOf("$");
			String subJobName = source.substring(start, stop);
			String id = resolve(subJobName);
			String ris = "#STATE:" + id + "$ERROR " + cause;
			return ris;
		} else if (source.equals("Concrete Execution Plan successfully built")
				|| source
						.equals("Concrete Execution Plan completed Successfully")) {
			return source;
		}
		return null;
	}

	private String resolveEdge(String source) {
		int start = source.indexOf(":");
		int stop = source.indexOf("$", 7);
		String subject = source.substring(start + 1, stop);
		Edge edge = infoEdgeMap.get(subject);
		if (edge == null) {
			return null;
		}
		String result = source.substring(0, start + 1);
		if (source.contains("ACTIVATE EDGE")) {
			start = stop;
			stop = source.indexOf("EDGE");
			result += edge.getID() + source.substring(start, stop);
		} else
			result += edge.getID() + source.substring(stop);
		return result;
	}

	private String resolveList(String source) {
		int start, stop;
		String name;
		if (!source.equals("")) {
			start = source.indexOf(":");
			stop = source.indexOf("[", 7);
			if (stop < 0) {
				return null;
			}
			start = source.indexOf("=", stop + 1);
			stop = source.indexOf("]", start);
			name = source.substring(start + 1, stop);
		} else {
			name = "";
		}
		Iterator<String> it = idSet.iterator();
		while (it.hasNext()) {
			String id = (String) it.next();
			Object o = elementMap.get(id);
			if (o instanceof Dataset) {
				Dataset dataset = (Dataset) o;
				if (dataset.getName().equals(o))
					return id;
			}
			if (o instanceof DataMiningSoftware) {
				DataMiningSoftware dms = (DataMiningSoftware) o;
				if (dms.getName().equals(name))
					return id;
			}
		}
		return null;
	}

	private String resolve(String subJobName) {
		Vector<String> nodes = taskNodeMap.get(subJobName);
		if (nodes != null) {
			for (String node : nodes) {
				return node;
			}
		}
		return null;
	}

	// FIXME EPMS: resolveDownload returns null id
	private String resolveDownload(String source,
			LinkedList<String> downloadOperation) {
		KDSUrl kdsUrl;
		try {
			kdsUrl = new KDSUrl(source);
		} catch (IllegalArgumentException e) {
			return null;
		}
		for (String id : idSet) {
			if (!downloadOperation.contains(id)) {
				if (id.startsWith("n")) {
					Object o = elementMap.get(id);
					if (o != null) {
						String href = null;
						if (o instanceof Dataset) {
							href = ((Dataset) o).getHref();
						} else if (o instanceof DataMiningSoftware) {
							href = ((DataMiningSoftware) o).getHref();
						}
						if (href != null && !href.isEmpty()
								&& href.equals(source)) {
							downloadOperation.add(id);
							return id;
						}
					}
				}
			}
		}
		for (String id : idSet) {
			if (!downloadOperation.contains(id)) {
				if (id.startsWith("n")) {
					Object o = elementMap.get(id);
					if (o != null) {
						String name = null;
						if (o instanceof Dataset) {
							name = ((Dataset) o).getName();
						} else if (o instanceof DataMiningSoftware) {
							name = ((DataMiningSoftware) o).getName();
						}
						if (name != null && !name.isEmpty()
								&& name.equals(kdsUrl.getName())) {
							downloadOperation.add(id);
							return id;
						}
					}
				}
			}
		}
		return null;
	}

	/**
	 * Subscribe the specified listener as consumer of notification 'KDSModel'
	 * topic of the specified KDS Service
	 * 
	 * @param listener
	 *            consumer of the notification
	 * @param kds
	 *            KDSPortType of the KDS Service whose the topic is a resource
	 *            property
	 */
	public static void subscribeToRAEMSModelTopic(EPMSListener listener,
			RAEMSPortType raems) throws Exception {

		// 1: set up the notification consumer
		// (the listener will be set up as a notification consumer)
		NotificationConsumerManager consumer = null;

		QName RAEMS_RP_MODEL = new QName(RAEMS_NAMESPACE, "RAEMSModel");

		// 2: set the 'listener' as a 'notification consumer'
		consumer = NotificationConsumerManager.getInstance();
		consumer.startListening();
		EndpointReferenceType consumerEPR = consumer
				.createNotificationConsumer(listener);

		// 3: create (and opportunetely set) the remote subscribe request
		// Create the request to the remote Subscribe() call
		Subscribe request = new Subscribe();
		// Must the notification be delivered using the Notify
		// operation?
		request.setUseNotify(Boolean.TRUE);
		// Indicate what the consumer client's EPR is
		request.setConsumerReference(consumerEPR);

		// 4: create the topic to be subscribed to
		// The TopicExpression specifies what topic we want to
		// subscribe to
		TopicExpressionType topicExpression = new TopicExpressionType();
		topicExpression.setDialect(WSNConstants.SIMPLE_TOPIC_DIALECT);
		topicExpression.setValue(RAEMS_RP_MODEL);
		request.setTopicExpression(topicExpression);

		// 6: send the subscribe request to the notification producer (KDS)
		// IMPORTANT: the consumer DASListener is subscribed to the 'KDS_Model'
		// topic
		raems.subscribe(request);
	}// subscribe

}
