/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.epms.service.impl;

import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceIdentifier;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;
import org.globus.wsrf.Topic;
import org.globus.wsrf.TopicList;
import org.globus.wsrf.TopicListAccessor;
import org.globus.wsrf.impl.ResourcePropertyTopic;
import org.globus.wsrf.impl.SimpleResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertySet;
import org.globus.wsrf.impl.SimpleTopicList;
import org.w3c.dom.Document;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class EPMSResource implements Resource, ResourceIdentifier,
        ResourceProperties, TopicListAccessor, Consts {

    // Resource Property set
    private ResourcePropertySet propSet;

    // Resource key. This uniquely identifies this resource.
    private Object              key;

    // Resource properties
    private ResourceProperty    modelRP;
    private ResourceProperty    planRP;
    private ResourceProperty    lastOpRP;

    // Topic list
    private TopicList           topicList;

    /**
     * Initializes RPs and returns a unique identifier for this resource
     */
    public Object initialize() throws Exception {
        // generate a key for the resource
        this.key = new Integer(hashCode());
        // create a property set instance
        this.propSet = new SimpleResourcePropertySet(RES_PROPERTIES);

        // create a 'modelRP' instance and a 'planRP' instance
        try {
            modelRP = new SimpleResourceProperty(RP_MODEL);
            modelRP.add(null);
            planRP = new SimpleResourceProperty(RP_PLAN);
            planRP.add(null);
            lastOpRP = new SimpleResourceProperty(RP_LASTOP);
            lastOpRP.add("NONE");
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage());
        }// try-catch

        // Configure the Topics: instantiate a topic list
        // and add 'modelRP' to the topic list
        this.topicList = new SimpleTopicList(this);
        modelRP = new ResourcePropertyTopic(modelRP);
        planRP = new ResourcePropertyTopic(planRP);
        lastOpRP = new ResourcePropertyTopic(lastOpRP);
        ((ResourcePropertyTopic) modelRP).setSendOldValue(true);
        ((ResourcePropertyTopic) planRP).setSendOldValue(true);
        ((ResourcePropertyTopic) lastOpRP).setSendOldValue(true);
        this.topicList.addTopic((Topic) modelRP);
        this.topicList.addTopic((Topic) planRP);
        this.topicList.addTopic((Topic) lastOpRP);
        this.propSet.add(modelRP);
        this.propSet.add(planRP);
        this.propSet.add(lastOpRP);
        return key;
    }

    public Document getModel() {
        return (Document) modelRP.get(0);
    }

    public void setModel(Document x) {
        modelRP.set(0, x);
    }

    public Document getPlan() {
        return (Document) planRP.get(0);
    }

    public void setPlan(Document x) {
        planRP.set(0, x);
    }

    public String getLastOp() {
        return (String) lastOpRP.get(0);
    }

    public void setLastOp(String x) {
        lastOpRP.set(0, x);
    }

    // Required by interface ResourceProperties
    public ResourcePropertySet getResourcePropertySet() {
        return this.propSet;
    }

    // Required by interface ResourceIdentifier/
    public Object getID() {
        return this.key;
    }

    // Required by interface TopicListAccessor
    public TopicList getTopicList() {
        return topicList;
    }

}
