/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.epms.service.impl;

/**
 * Represents a KDS URL. The address must be composed in this way:
 * "kds://hostname/filename.extension"
 * 
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class KDSUrl implements Consts {

    private String host;
    private String metadata;
    private String name;

    /**
     * @param address
     * @throws IllegalArgumentException
     *             if address does not comply with the KDS URL structure
     */
    public KDSUrl(String address) throws IllegalArgumentException {
        if (address == null || !address.startsWith(KDS_PROTOCOL)) {
            throw new IllegalArgumentException();
        }
        int k = address.indexOf("/", KDS_PROTOCOL.length());
        if (k == -1) {
            throw new IllegalArgumentException();
        }
        host = address.substring(KDS_PROTOCOL.length(), k);
        metadata = address.substring(k + 1);
        k = metadata.lastIndexOf(".");
        if (k == -1) {
            throw new IllegalArgumentException();
        }
        name = metadata.substring(0, k);
    }

    public String getName() {
        return name;
    }

    public KDSUrl(String host, String metadata) {
        this.host = host;
        this.metadata = metadata;
    }

    public String getHostName() {
        return host;
    }

    public void setHostName(String host) {
        this.host = host;
    }

    public String getMetadataFileName() {
        return metadata;
    }

    public void setMetadataFileName(String metadata) {
        this.metadata = metadata;
    }

    public String getURL() {
        return KDS_PROTOCOL + this.host + "/" + this.metadata;
    }

    public boolean equals(Object o) {
        if (!(o instanceof KDSUrl)) {
            return false;
        }
        KDSUrl url = (KDSUrl) o;
        return url.host.equals(this.host) && url.metadata.equals(this.metadata);
    }

}
