/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.epms.service.impl;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class Utils implements Consts {

	public static String HOST_NAME;
	public static String HOST_IP;
	private static final String DATE_PATTERN = "yyyy.MM.dd HH:mm:ss";
	private static final SimpleDateFormat DATE_FORMATTER = new SimpleDateFormat(
			DATE_PATTERN);

	public static void print(String s) {
		String dateAndTime = DATE_FORMATTER.format(new Date());
		System.out.println(dateAndTime + " " + SERVICE_ACRONYM + " " + s);
	}

	static {
		try {
			HOST_NAME = (InetAddress.getLocalHost()).getCanonicalHostName();
			if (HOST_NAME == null || HOST_NAME.isEmpty()) {
				print("Error getting hostname from localhost (host name is empty)");
			}
			HOST_IP = (InetAddress.getLocalHost()).getHostAddress();
			if (HOST_IP == null || HOST_IP.isEmpty()) {
				print("Error getting host address from localhost (host address is empty)");
			}
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
	}

	public static void debugInfo() {
		System.out
				.println("*******************************************************");
		System.out.println();
		System.out.println("Host name and address = " + HOST_NAME + " "
				+ HOST_IP);
		System.out.println();
		System.out.println("User account name = "
				+ System.getProperty("user.name"));
		System.out.println("User home directory = "
				+ System.getProperty("user.home"));

		System.out
				.println("User working directory (i.e. the the directory where the globus container is started from) = "
						+ System.getProperty("user.dir"));

		System.out.println();
		System.out.println("Path of extension directory or directories = "
				+ System.getProperty("java.ext.dirs"));
		System.out.println("List of paths to search when loading libraries = "
				+ System.getProperty("java.library.path"));
		System.out.println("Java class path = "
				+ System.getProperty("java.class.path"));

		System.out.println();
		System.out.println("Java Virtual Machine implementation name = "
				+ System.getProperty("java.vm.name"));
		System.out.println("Java Runtime Environment version = "
				+ System.getProperty("java.version"));
		System.out.println("Java Runtime Environment vendor = "
				+ System.getProperty("java.vendor"));
		System.out.println();
		System.out
				.println("*******************************************************");
		System.out.println();
		System.out.println("Press any key to continue...");

		try {
			System.in.read();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static void printDocument(Document doc) throws IOException {
		OutputFormat format = new OutputFormat(doc);
		format.setIndenting(true);
		XMLSerializer serializer = new XMLSerializer(System.out, format);
		serializer.serialize(doc);
	}
}
