/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.epms.service.impl.tag;

import java.util.Vector;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class Attribute {
    private String name = null;
    private String type = null;
    private SubType subtype = null;
    private Vector<Parameter> list;

    public Attribute() {
        name = "";
        type = "";
        list = new Vector<Parameter>();
    }

    public Attribute(String name, String type) {
        this.name = name;
        this.type = type;
        list = new Vector<Parameter>();
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getType() {
        return type;
    }

    public void setSubType(SubType subtype) {
        this.subtype = subtype;
    }

    public SubType getSubType() {
        return subtype;
    }

    public boolean addParameter(Parameter parameter) {
        return list.add(parameter);
    }

    public boolean removeParameter(Parameter parameter) {
        return list.remove(parameter);
    }

    public int parameterSize() {
        return list.size();
    }

    public Parameter getParameter(int index) {
        return list.get(index);
    }

    public String toString() {
        String ris = "<Attribute ";
        ris += "name=\"" + name + "\" ";
        ris += "type=\"" + type + "\">";
        ris += "\n";
        if (subtype != null) {
            ris += subtype.toString();
            ris += "\n";
        }
        if (list.size() != 0)
            for (int i = 0; i < list.size(); i++) {
                Parameter parameter = (Parameter) list.get(i);
                ris += parameter.toString();
                ris += "\n";
            }
        ris += "</Attribute>";
        return ris;
    }

    public boolean equals(Object o) {
        Attribute attribute;
        if (o instanceof Attribute)
            attribute = (Attribute) o;
        else
            return false;
        if (!(this.getName().equals(attribute.getName()))
                && (this.getType().equals(attribute.getType()))
                && (this.getSubType().equals(attribute.getSubType())))
            return false;
        boolean equal = true;
        for (int i = 0; i < this.parameterSize() && equal; i++) {
            if (this.getParameter(i) != null
                    && attribute.getParameter(i) != null) {
                if (!(this.getParameter(i).equals(attribute.getParameter(i))))
                    equal = false;
            } else
                equal = false;
        }
        return equal;
    }

}
