/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.epms.service.impl.tag;

import java.util.Vector;

/**
 * @author Eugenio Cesario <cesario@icar.cnr.it>
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class Computation {
    private Vector<Input> listInput;
    private Program program;
    private Vector<Output> listOutput;

    public Computation() {
        listInput = new Vector<Input>();
        listOutput = new Vector<Output>();
        program = null;
    }

    public boolean addInput(Input input) {
        return listInput.add(input);
    }

    public boolean removeInput(Input input) {
        return listInput.remove(input);
    }

    public Input getInput(int index) {
        return listInput.get(index);
    }

    public int sizeInput() {
        return listInput.size();
    }

    public boolean addOutput(Output output) {
        return listOutput.add(output);
    }

    public boolean removeOutput(Output output) {
        return listOutput.remove(output);
    }

    public Output getOutput(int index) {
        return listOutput.get(index);
    }

    public int sizeOutput() {
        return listOutput.size();
    }

    public void setProgram(Program program) {
        this.program = program;
    }

    public Program getProgram() {
        return program;
    }

    public String toString() {
        String ris = "<Computation>";
        ris += "\n";
        if (listInput.size() != 0)
            for (int i = 0; i < listInput.size(); i++) {
                Input input = (Input) listInput.get(i);
                ris += input.toString();
                ris += "\n";
            }
        if (program != null) {
            ris += program.toString();
            ris += "\n";
        }
        if (listOutput.size() != 0)
            for (int i = 0; i < listOutput.size(); i++) {
                Output output = (Output) listOutput.get(i);
                ris += output.toString();
                ris += "\n";
            }
        ris += "</Computation>";
        return ris;
    }

}
